#pragma once
#include "BasisFunktion.h"
#include "glm/glm.hpp"
using glm::vec3;
using glm::vec4;

 /*!
  * This class creates/calculates/recalculates B-Splines
  */ 
class BSpline
{
public:
	/*!
	  * Constructor to create an BSpline object
	  * @param basisFunktion to get the weights for the B-Spline calculation
	  * @param controlPolygonX the x-values of the control points from the  B-Spline
	  * @param controlPolygonY the y-values of the control points from the  B-Spline
	  * @param controlPolygonZ the z-values of the control points from the  B-Spline
	  * @param controlPolygonSize the number of control points
	  * @param id the id of the adjacency edge
	  */ 
	BSpline(BasisFunktion* basisFunktion, float* controlPolygonX, float* controlPolygonY, float* controlPolygonZ, int controlPolygonSize, float id);
	/*!
	* Destructor
	*/
	~BSpline(void);

	/*!
	* 	Method to recalculate the  B-Spline with a given beta (bundling strength)
	* @param beta bundling strength
	*/ 
	void recalculate(float beta);
	/*!
	*	return a pointer to the first element of a array containing vec3 of the calculated  B-Spline points for the line-strip
	* @return pointer to array containing the xyz-positions of the calculated Values
	*/	
	vec3* getValues();
	/*!
	* return the squared length of the  B-Spline
	* @return a float containing the length
	*/	
	float getLength() const;
	/*!
	* this method is called to draw the  B-Spline
	*/	
	void draw();

	int GetID();
	int GetVectorPos();
	void SetID(int id);
	void SetVectorPos(int id);

	void UpdatePosition();
	void UpdateIDs(float id);
	void UpdateColor(vec4 startcolor, vec4 endcolor);
private:
	unsigned int vbohierarchylines_vert;
	unsigned int vbohierarchylines_color;
	unsigned int vboindex_picking_ind;

private:
	/*!
	* pointer to the basis function influencing the B-Spline
	*/	
    BasisFunktion* basisFunktion;
	/*!
	* Number of control points of the  B-Spline
	*/	
    int controlPolygonSize;
	/*!
	* Number of points which are calculated to generate a line-strip
	*/
    int splinePoints;
	/*!
	* pointer to float[] with the initial x-Values of the ControlPolygon 
	*/
    float* initialX;
	/*!
	* pointer to float[] with the initial y-Values of the ControlPolygon
	*/
    float* initialY;
	/*!
	* pointer to float[] with the initial z-Values of the ControlPolygon
	*/
    float* initialZ;
	/*!
	* pointer to float[] with the streightende x-Values of the ControlPolygon
	*/
    float* bundlingX;
	/*!
	* pointer to float[] with the streightende y-Values of the ControlPolygon
	*/
    float* bundlingY;
	/*!
	* pointer to float[] with the streightende z-Values of the ControlPolygon
	*/
    float* bundlingZ;
	/*!
	* the suared length of the BSpline
	*/
	float length;

	float m_id;
	float m_arraypos;

	/*! 
	* pointer to vec3[] with the xyz-coordinates of the calculated B-Spline
	*/
	vec3* values;
	/*!
	* calculates the values for the BSplinePoints an fill the Values array
	*/
	void calculate();
	/*!
	* calculates the squared length of the  B-Spline to sort them by length
	*/
	void calculateLength();

};