#pragma once
#include "glm/glm.hpp"

using glm::vec3;
using glm::mat4;

class Camera
{
public:
	/*! The constructor for a perspective camera. */
	Camera(float zNear, float zFar, float fovyAngle, float aspect);
	/*! The constructor for a orthogonal camera. */
	Camera(float left, float right, float bottom, float top, float near, float far);
	/*! Destructor */
	~Camera();

	/*! Returns the Viewingmatrix. */
	mat4 getViewingMatrix() const;
	/*! Sets the Viewingmatrix. */
	void setViewingMatrix(mat4 matrix);
	/*! Returns the Projectionmatrix. */
	mat4 getProjectionMatrix();
	/*! Returns the Position of the Camera. */
	const vec3 getPosition() const;
	
	/*! Builds a look at view matrix. */
	void lookAt(vec3 position, vec3 upVector, vec3 lookAtVector);
	/*! Sets the Camera to a Perspectiv Camera. */
	void perspective(float zNear, float zFar, float fovy, float aspect);
	/*! Sets the Camera to a Orthogonal Camera. */
	void orthogonal(float left, float right, float bottom, float top, float near, float far);

	/*! 
	 * Sets the screen width. 
	 * @param width The width of the screen.
	 */
	void setScreenWidth(float width);

	/*! 
	 * Returns the screen width. 
	 * @return The width of the screen.
	 */
	float getScreenWidth();

	/*! 
	 * Sets the screen height. 
	 * @param height The height of the screen.
	 */
	void setScreenHeight(float height);

	/*! 
	 * Returns the screen height. 
	 * @return The height of the screen.
	 */
	float getScreenHeight();

	/*! 
	 * Sets the nearplane. 
	 * @param near The nearplane.
	 */
	void setNearplane(float near);

	/*! 
	 * Sets the farplane.
	 * @param far The farplane.
	 */
	void setFarplane(float far);

	/*! 
	 * Updates the orthogonal view of the camera depending on the current 
	 * screenWidth, screenHeight, nearplane, farplane and zoom factor.
	 */
	void updateOrthogonalView();

	/*! 
	 * Updates the perspective view of the camera depending on the current 
	 * screenWidth, screenHeight, nearplane, farplane, fovy and zoom factor.
	 */
	void updatePerspectiveView();

	/*! 
	 * Sets the zoomFactor to the given value.
	 * @param zoom The zoom factor. 
	 */
	void setZoomFactor(float zoom);

	/*! Returns the zoomFactor of the camera. */
	float getZoomFactor();

	/*! Zooming in the camera. */
	void zoomIn();

	/*! Zooming out the camera. */
	void zoomOut();

private:
	/*! The width of the screen. */
	float screenWidth;
	/*! The height of the screen. */
	float screenHeight;
	/*! The nearplane. */
	float nearplane;
	/*! The farplane. */
	float farplane;
	/*! The field of view angle, in degrees, in the y direction. */
	float fovy;
	/*! The zoom factor of the camera. */
	float zoomFactor;

	/*! The view matrix of the camera. */
	mat4 viewingMatrix;
	/*! The projection matrix of the camera. */
	mat4 projectionMatrix;
	
};