#pragma once
#include <glew.h >
#include <windows.h>
#include <stdlib.h>
#include <io.h>
#include <stdio.h>
#include <vector>

//!  FBO class. 
/*!
 * Diese Klasse dient zur Verwaltung des FBO (Frame Buffer Objects). Es erzeugt einen FBO, welcher die Farbwerte speichert (Depthmap wird nicht bentzt).
 * Es werden Funktionen zur Erzeugung (durch Konstruktor), zum Binding des FBOs und Textur und zum Lschen (Dekonstruktor) zur Verfgung gestellt.
 * @author Michael Beham
 * @date 4.12.2010 
*/

class FBO
{
public:
/**  Kontstruktor: Erstellt den neuen FBO und anschlieend auf Fehler berprft (bzw. war die Erstellung erfolgreich).
  *  @param[in] g_iWidth Die Breite des neuen FBOs.
  *  @param[in] g_iHeight Die Hhe des neuen FBOs. 
  */

	FBO(int g_iWidth, int g_iHeight, int numTex)
	{
		bool bit32 =true;
		GLenum FBOstatus;
		
		glGenTextures(1, &depthbuffer1);
		glBindTexture(GL_TEXTURE_2D, depthbuffer1);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
		glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );	
		glTexImage2D( GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, g_iWidth, g_iHeight, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
		glBindTexture(GL_TEXTURE_2D, 0);
	
		tex1.resize(numTex);
		for(unsigned int i = 0; i < tex1.size(); i++)
		{
			glGenTextures(1,&tex1[i]);
			glBindTexture(GL_TEXTURE_2D, tex1[i]);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	
			// Remove artefa3ct on the edges of the shadowmap
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
			glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );

			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA32F_ARB, g_iWidth, g_iHeight, 0, GL_RGBA, GL_FLOAT, 0);
			glBindTexture(GL_TEXTURE_2D, 0);
		}

		glGenFramebuffersEXT(1, &fbo1);
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo1);
		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT ,GL_TEXTURE_2D, depthbuffer1, 0);	

		buffers.resize(tex1.size());
		for(unsigned int i = 0; i < tex1.size(); i++)
		{
			glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT+i,GL_TEXTURE_2D, tex1[i], 0);
			buffers[i] = GL_COLOR_ATTACHMENT0_EXT+i;
		}		
		
	
		FBOstatus = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
		if(FBOstatus != GL_FRAMEBUFFER_COMPLETE_EXT) CheckFBO(FBOstatus);
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	};

/**  Dekontstruktor: Lscht den FBO und gibt seine Ressourcen wieder frei (samt Texturen,etc.)
  */
	~FBO()
	{
		glDeleteFramebuffersEXT(1, &fbo1);
		glDeleteRenderbuffersEXT(1, &depthbuffer1);
		for(unsigned int i = 0; i < tex1.size(); i++)	glDeleteTextures(1, &tex1[i]);
		tex1.clear();
	};

/**  CheckFBO: Diese Funktion analysiert den Status des FBOs und gibt im Fehlerfall eine Fehlermeldung aus.
  *  @param[in] FBOstatus Status des FBOs.
  */
	void CheckFBO(GLenum FBOstatus)
	{
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT)			printf("Incomplete attachment\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT)	printf("Missing attachment\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT)	printf("Incomplete dimensions\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT)			printf("Incomplete formats\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT)				printf("Incomplete draw buffer\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT)			printf("Incomplete read buffer\n");
		if(FBOstatus == GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT)			printf("GL_FRAMEBUFFER_COMPLETE_EXT fr Blur Effekt konnte ned benutzt werden\n");
		if(FBOstatus == GL_FRAMEBUFFER_UNSUPPORTED_EXT)						printf("Framebufferobjects unsupported\n");
	};

/**  Bind: Bindet den FBO.
  */
	void Bind()
	{
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fbo1);	//FRAMEBUFFEROBJECT START!!!
	};

/**  Unbind: Entbindet den FBO.
   */
	void Unbind()
	{
		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
	};

/**  BindColorAttachementAtTextureAndEnable: Diese Funktion bindet die Textur des FBOs auf eine gewnschte Texturebene.
  *  @param[in] i Ebene, welche die Textur abgebildet wird.
  */
	void BindColorAttachementAtTextureAndEnable(unsigned int i)
	{
		glActiveTexture(GL_TEXTURE0+i);
		glBindTexture(GL_TEXTURE_2D, tex1[0]);
		glEnable(GL_TEXTURE_2D);
	};

 /*!
  * Diese methode setzt den Tiefenbuffer auf eine definierte Textur
  * @param i [int] Textur, auf der der Tiefenbuffer gezeichnet werden soll
  */
	void BindDepthBufferAtTextureAndEnable(unsigned int i)
	{
		glActiveTexture(GL_TEXTURE0+i);
		glBindTexture(GL_TEXTURE_2D, depthbuffer1);
		glEnable(GL_TEXTURE_2D);
	};

	/*!
	 * Diese Methode ermglicht gleichzeitiges Rendern in mehreren Buffern
	 */
	void DrawBuffers()
	{
		glDrawBuffers(buffers.size(), &buffers[0]);
	}

	GLuint fbo1;
	GLuint depthbuffer1;
	std::vector<GLuint> tex1;
	std::vector<GLenum> buffers;

};