#pragma once
#include "Vertex.h"
#include "HierarchyEdge.h"
#include "RelationEdge.h"
#include "Hierarchy.h"

#include <iostream>
#include <iomanip>
#include <string>

#include <boost/ptr_container/ptr_sequence_adapter.hpp>

#include <boost/ptr_container/ptr_map.hpp>
#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/assign/ptr_map_inserter.hpp>  // for 'ptr_map_insert()'
#include <boost/shared_ptr.hpp>
#include <memory>
#include <vector>
#include <map>
#include <algorithm>

using namespace std;
using namespace boost;
using namespace boost::assign;

typedef boost::shared_ptr<Vertex> VertexPtr;
typedef boost::shared_ptr<HierarchyEdge> HierarchyPtr;
typedef boost::shared_ptr<RelationEdge> RelationPtr;

/*!
 * This class provides methods for creating, editing and deleting graphs.
 * Is possible to:
 * @li add nodes
 * @li add edges (relation and hierarchy)
 * @li deleting edges
 * @li deleting nodes
 * @li add and removing hierarchies
 * @li calculating the LCA, path to root and more
 * @author Michael Beham
 * @date 6.6.2011
 */
class Graph
{
public:
	/*!
	 * standard constructor
	 */
	Graph(void);
	/*!
	 * standard deconstructor
	 */
	~Graph(void);

	/*!
	 * This method adds a new node to the graph
	 * @param vert [VertexPtr] new node
	 */
	void AddVertex(VertexPtr vert);
	/*!
	 * This method returns the node corresponding to the passed id
	 * @param id [unsigned int] id
	 * @return [VertexPtr] node
	 */
	VertexPtr GetVertexWithID(unsigned int id);
	/*!
	 * This method returns the number of vertices.
	 * @return [int] number of vertices
	 */
	int GetNumberOfVertices();
	/*!
	 * This method removes the node corresponding to the passed id.
	 * @param id [unsigned int] id
	 */
	void RemoveVertexWithID(unsigned int id);
	/*!
	 * This method removes the passed node.
	 * @param vert [Vertex] Node to delete.
	 */
	void RemoveVertex(Vertex& vert);

	/*!
	 * This method set the node corresponding to the passed id as root.
	 * @param id [int] id of the root.
	 */
	void SetRoot(int id);
	/*!
	 * This method returns the root.
	 * @param[VertexPtr]  root.
	 */
	VertexPtr GetRoot();

	/*!
	 * This method returns a hierarchy edge to the corresponding id
	 * @param count [int] id of the edge
	 * @return [RelationPtr] edge to add
	 */
	HierarchyPtr GetHierarchyEdgeWithID(unsigned int id);
	/*!
	 * This method adds a hierarchy edge to the graph.
	 * @return [RelationPtr] edge to add
	 */
	void AddHierarchicEdge(HierarchyPtr edge);
	/*!
	 * This method returns the number of hierarchy edges
	 * @return [RelationPtr] number of hierarchy edges
	 */
	unsigned int GetNumberOfHiearchicEdges();
	/*!
	 * This method returns the hierarchy edge at a passed position.
	 * @param count [int] position of the edge
	 * @return [RelationPtr] relation id
	 */
	HierarchyPtr GetHierarchyEdgeAt(unsigned int at);

	/*!
	 * This method returns a relation edge to the corresponding id
	 * @param count [int] id of the edge
	 * @return [RelationPtr] edge to add
	 */
	RelationPtr GetRelationEdgeWithID(unsigned int id);
	/*!
	 * This method adds a relation edge to the graph.
	 * @return [RelationPtr] edge to add
	 */
	void AddRelationEdge(RelationPtr edge);
	/*!
	 * This method returns the number of relation edges
	 * @return [RelationPtr] number of relation edges
	 */
	unsigned int GetNumberOfRelationEdges();
	/*!
	 * This method returns the Relation edge at a passed position.
	 * @param count [int] position of the edge
	 * @return [RelationPtr] relation id
	 */
	RelationPtr GetRelationEdgeAt(unsigned int at);

	/*!
	 * This method sets the number of hierarchies .
	 * @param count [int] number of hierarchies
	 */
	void SetNumberOfHierarchies(int count);
	/*!
	 * This method returns the number of hierarchies
	 * @return [int] number of hierarchies
	 */
	int GetNumberOfHierarchies();
	
	/*!
	 * This method adds a hierarchy level to the graph
	 * @param hierarchy[Hierarchy] hierarchy level
	 */
	void AddHierarchy(Hierarchy hierarchy);
	/*!
	 * This method updates a adds a hierarchy edge to the graph
	 * @param level[int] hierarchy to update
	 * @param hierarchy[Hierarchy] hierarchy level
	 */
	void SetHierarchyAt(int level, Hierarchy hierarchy);
	/*!
	 * This method returns the hierarchy
	 * @param level[int] selected hierarchy
	 * @return[Hierarchy] hierarchy level
	 */
	Hierarchy GetHierarchyAt(int level);
	/*!
	 * This method removes a hierarchy
	 * @param level[int] hierarchy to remove
	 */
	void RemoveHierarchy(int level);

	/*!
	 * This method converts a list of IDs to a list of VertexPtr
	 * @param ids [std::vector<unsigned int>] list of id
	 * @return [std::vector<unsigned int>] list of VertexPtr
	 */
	std::vector<VertexPtr> GetVertexPtrVectorFromIDVector(std::vector<unsigned int> ids);

	/*!
	 * This method calculates all needed information from the tree
	 */
	void Calculate();
	/*!
	 * This method calculates the hierarchy level of each node
	 */
	void CalculateHierarchy();
	/*
	 * This method calculates a path to the root node for all nodes.
	 */
	void CalculatePathToRoot();
	/*
	 * This method calculates a path to the root
	 * @param actPath [VertexPtr] act path
	 * @param path [unsigned int] nodes 
	 */
	void CalculatePathToRoot(VertexPtr actPath, std::vector<unsigned int> path);

	/*!
	 * This method calculates the LCA for a passed relation edge
	 * @param edge [RelationPtr] relation edge
	 * @return [unsigned int] node id list
	 */
	std::vector<unsigned int> CalculatePathOverLCA(RelationPtr edge);
	/*!
	 * This method calculates the LCA form a passed start note to a passed end node.
	 * @param start [RelationPtr] start node
	 * @param end [RelationPtr] end node
	 * @return [unsigned int] node id list
	 */
	std::vector<unsigned int> CalculatePathOverLCA(VertexPtr start, VertexPtr end);

	/*!
	 * This method returns all nodes without a child node.
	 * @return [unsigned int] list of all vertices without a child
	 */
	std::vector<VertexPtr> GetAllVerticesWithoutChild();
protected:
	std::map<unsigned int, VertexPtr> m_vertices;
	std::map<unsigned int, HierarchyPtr> m_hierarchyedges;
	std::map<unsigned int, RelationPtr> m_relationedge;
	VertexPtr m_pkRoot;

	vector<Hierarchy> m_hierarchy;
};
