#pragma once
#include "common.h"
#include "glm/glm.hpp"

using glm::vec3;
using glm::vec4;
using glm::mat4;

/*!
 * This class creates a light. 
 * We support a standard point a light, a spot light, and a conic light
 * @author Michael Beham
 * @date 3.6.2011
 */
class Light
{
public:
	/*!
	 * Defines the lighttype
	 */
	enum LightType
	{
		LT_POINT = 0,
		LT_CONIC = 1,
		LT_SPOTLIGHT = 2,
		LT_NONE
	};

public:
	/*!
	 * standard constructor
	 */
	Light(void);
	/*!
	 * standard deconstructor
	 */
	~Light(void);

	/*!
	 * Set the light on
	 */
	void On();
	/*!
	 * Set the light off
	 */
	void Off();
	/*!
	 * Is the light on?
	 */
	bool IsOn();

	/*!
	 * Set shadow on
	 */
	void ShadowOn();
	/*!
	 * Set shadow off
	 */
	void ShadowOff();
	/*!
	 * Returns the shadow state
	 * @return [bool] state of shadow
	 */
	bool IsShadowOn();
	
	/*!
	 * This method sets the element, which should be a part of the shadow.
	 * The input follows the idea of a bit array.
	 * @li 1... nodes should be a part of shadow
	 * @li 2... hierarchy edges should be a part of shadow
	 * @li 4.... relation edges should be a part of shadow
	 * @li 8.... grid should be a part of shadow
	 * @param elements [int] elements which should be draw.
	 */
	void SetShadowElements(int elements);
	/*!
	 * Returns true, if the vertices should be a part of the shadow.
	 * return [bool] true... if vertices should be a part of the shadow
	 */
	bool AreVerticesInShadow();
	/*!
	 * Returns true, if the the grid should be a part of the shadow.
	 * return [bool] true... if the grid should be a part of the shadow
	 */
	bool AreGridsInShadow();
	/*!
	 * Returns true, if the hierarchy edge should be a part of the shadow.
	 * return [bool] true... if hiearchy edge should be a part of the shadow
	 */
	bool AreHierarchyEdgeInShadow();
	/*!
	 * Returns true, if the hierarchy edge should be a part of the shadow.
	 * return [bool] true... if hiearchy edge should be a part of the shadow
	 */
	bool AreRelationEdgeInShadow();

	/*!
	 * This method sets the light type
	 * @param position [vec3] position
	 */
	void SetType(int type);
	/*!
	 * This method sets the light position
	 * @param position [vec3] position
	 */
	void SetPosition(vec3 position);
	/*!
	 * This method sets the spot direction
	 * @param spotdirection [vec3] Spotdirection
	 */
	void SetSpotDirection(vec3 spotdirection);
	/*!
	 * This method sets the light radius
	 * @param radius [float] radius of the light
	 */
	void SetRadius(float radius);
	/*!
	 * Set the light attentuate
	 * @param attentuate [float] light attentuate
	 */
	void SetAttentuate(float attentuate);

	/*!
	 * Return the light type
	 * @return [float] return the light type
	 */
	float GetType();
	/*!
	 * Returns the position of the light
	 * @return [vec3] position of the light
	 */
	vec3 GetPosition();
	/*!
	 * Return the Spot direction of the light
	 * @return [vec3] spot direction
	 */
	vec3 GetSpotDirection();
	/*!
	 * Returns the radius of the light
	 * @return [float] return the light
	 */
	float GetRadius();
	/*!
	 * Returns attentuate of the light
	 * @return [float] returns the attentuate
	 */
	float GetAttentuate();

private:
	bool m_shadowon;
	bool m_on;
	int m_actType;
	vec3 m_position;
	vec3 m_spotdirection;
	float m_radius;
	float m_attentuate;

	bool m_shadowvertices;
	bool m_shadowhierarchyedge;
	bool m_shadowrelationedge;
	bool m_shadowgrid;

};
