#pragma once
#include "Renderer.h"
#include "Shader.h"
#include "FBO.h"
#include "Camera.h"
#include "Light.h"
#include "View.h"
#include "Image.h"

#include <glew.h>
#include <QGLWidget>
#include <QtOpenGL>
#include <QColor>
#include <QGLContext>
#include <QGLFormat>

/*!
 * This method implements the renderer
 * @author Michael Beham
 * @date 6.6.2011
 */
class RenderImplementation :	public Renderer
{
public:
	/*!
	* standard constructor
	*/
	RenderImplementation(void);
	/*!
	 * standard deconstructor
	 */
	~RenderImplementation(void);

	/*!
	 * This method initialize the renderer
	 */
	void Initialize();
	/*!
	 * This method creates the rendering data for all layouts
	 * @parm view [View**] layout data
	 */
	void CreateObjects(View** view);
	/*!
	 * This method draws the actual frame
	 * @param camera [Camera*] scene camera
	 * @param light [Light*] scene light source
	 */
	void DrawObjects(Camera* camera, Light* light);

	/*!
	 * This method draws the grid
	 */
	void DrawGrid();
	/*!
	 * This method draws all vertices
	 */
	void DrawVertices();
	/*!
	 * This method draws all vertices
	 */
	void DrawAllVertices();
	/*!
	 * This method draws the selected vertex
	 * param v [VertexPtr] Node to draw
	 */
	void DrawSelectedVertex(VertexPtr v);
	/*!
	 * This method draws all nodes without the passed.
	 * @param v [VertexPtr] node, which shouldn't draw
	 */
	void DrawNotSelectedVertices(VertexPtr v);
	/*!
	 * This method draws the relation edges
	 */
	void DrawRelationEdges();
	/*!
	 * This method draws the hierarchy edges
	 */
	void DrawHierarchyEdges();
	/*!
	 * This method draws the hierarchy edges
	 * @param noshadow [bool] true... if it's not a shadow pass
	 */
	void DrawHierarchyEdges(bool noshadow);
	/*!
	 * This method draws textures to scren
	 * @tex [int] texture 1
	 * @tex2 [int] texture 2
	 * @tex3 [int] texture 3
	 * @tex4 [int] texture 4
	 * @tex5 [int] texture 5
	 * @tex6 [int] texture 6
	 * @tex7 [int] texture 7
	 */
	void DrawScreenAlignedQuad(int tex, int tex2, int tex3, int tex4, int tex5,int tex6,int tex7);

	/*!
	 * This method enables the camera
	 */
	void BeginCamera();
	/*!
	 * This method enables the camera
	 * @param cam [Camera*] camera
	 */
	void BeginCamera(Camera* cam);
	/*!
	 * This method disables the camera
	 */
	void EndCamera();
	/*!
	 * This method binds textures
	 * @param tex  [int] texture1
	 * @param tex2 [int] texture2
	 * @param tex3 [int] texture3
	 * @param tex4 [int] texture4
	 * @param tex5 [int] texture5
	 * @param tex6 [int] texture6
	 * @param tex7 [int] texture7
	 */
	void BeginTextures(int tex, int tex2, int tex3, int tex4, int tex5,int tex6,int tex7);
	/*!
	 * This method unbind all textures
	 */
	void EndTextures();	

	Camera* camera;

public:
	//FONT
	/*!
	 * This method loads the font
	 */
	void LoadFont();
	/*!
	 * This method deletes the front
	 */
	void UnloadFont();
	/*!
	 * This method draws a word at a specified position
	 * @param x [GLint] screen x position
	 * @param y [GLint] screen y position
	 * @param string [std::string] word to draw
	 * @param set [int] font
	 * @param scale [float] scale of the font
	 */
	void DrawFont(GLint x, GLint y, std::string string, int set,float scale);	// Where The Printing Happens
	/*!
	 * This method draws the HUD
	 * @param time [int] time for calculating the actual frame
	 */
	void DrawHUD(int time);
	/*!
	 * This method returns the actual modelview matrix
	 * @return [Matrix] model view matrix
	 */
	Matrix GetModelViewMatrix();
	/*!
	 * This method returns the actual projection matrix
	 * @return [Matrix] projection matrix
	 */
	Matrix GetProjectionMatrix();

public:
	/*!
	 * This method performs picking
	 * @param curserX [int] x position of the curser
	 * @param curserY [int] y position of the curser
	 */
	void Picking(int curserX, int curserY);

	/*!
	 * This method updates the color of all spheres
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 */
	virtual void UpdateSphereColor(vec3 color, float alpha);
	/*!
	 * This method updates the color of all hierarchy edges
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 */
	virtual void UpdateHierarchyEdgeColor(vec3 color, float alpha);
	/*!
	 * This method updates the color of all hierarchy edges
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 * @param endcolor [vec3] end color of the edge
	 * @param ebdalpha [float] end alpha of the edge
	 */
	void UpdateHierarchyEdgeColorAt(vec3 color, float alpha, unsigned int actView);
	/*!
	 * This method updates the color of a given hierarchy edge
	 * @param at [unsigned int] selected edge
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 * @param endcolor [vec3] end color of the edge
	 * @param ebdalpha [float] end alpha of the edge
	 */
	void UpdateHierarchyEdgeColorSelectionAt(vec3 color, float alpha, vec3 sele_color, float sele_alpha, unsigned int at);
	/*!
	 * This method updates the color of a given relation edge
	 * @param at [unsigned int] selected edge
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 * @param endcolor [vec3] end color of the edge
	 * @param ebdalpha [float] end alpha of the edge
	 */
	void UpdateRelationEdgeColor(vec3 color, float alpha, vec3 sele_color, float sele_alpha, unsigned int at);
	/*!
	 * This method updates the color of all relation edges
	 * @param color [vec3] start color of the edge
	 * @param alpha [float] start alpha of the edge
	 * @param endcolor [vec3] end color of the edge
	 * @param ebdalpha [float] end alpha of the edge
	 */
	virtual void UpdateRelationEdgeColor(vec3 color, float alpha, vec3 endcolor, float endalpha);
	/*!
	 * This method updates the selection color
	 * @param color [vec3] color 
	 * @param alpha [float] alpha
	 */
	virtual void UpdateSelectionColor(vec3 color, float alpha);
private:
	/*!
	 * This method draws all vertices
	 * @param lightmatrix [Matrix] lightmatrix
	 */
	void DrawVerticesPass(Matrix lightmatrix);
	/*!
	 * This method draws the relation for summed average and weighting average method
	 */
	void DrawRelationEdgePass();
	/*!
	 * This method draws the relation edges
	 */
	void DrawRelationEdgePassMethod1();
	/*!
	 * This method draws the relation edges
	 */
	void DrawRelationEdgePassMethod2();
	/*!
	 * This method draws the relation edges for picking in the fbo 
	 */
	void DrawRelationEdgePassMethod3();
	/*!
	 * This method calculates the shadow
	 * @param lightmatrix [Matrix] lightmatrix for calculating the shadow.
	 */
	void DrawShadowPass(Matrix& m);
	/*!
	 * This method draws the shadow
	 * @param lightmatrix [Matrix] lightmatrix for calculating the shadow.
	 */
	void DrawShadow(Matrix lightmatrix);
	/*!
	 * This method draws the result from the fbo and perform additional blening.
	 */
	void DrawResult();
	/*!
	 * This method updates the lightsource.
	 */
	void UpdateLight();
	/*!
	 * This method updates and manages the selection
	 */
	void UpdateSelection();

	int rendermode;
	
	Camera* cam_shadow;	

	//Shader
	ShaderObj* shaderobj_shadow;
	Shader* shadowshader;

	Shader* lightingshader;
	ShaderObj* shaderobj_lighting;

	ShaderObj* shaderobj;
	Shader* lightningshader;

	Shader* initweightingshader;	
	ShaderObj* shaderobj_weighting;

	ShaderObj* shaderobj_finalweighting;
	Shader* finalweightingshader;

	ShaderObj* shaderobj_blending;
	Shader* initblendingshader;

	Shader* initsumshader;
	ShaderObj* shaderobj_sum;
	Shader* finalsumshader;
	ShaderObj* shaderobj_finalsum;

	//FBO
	FBO* fbo_opacity;
	FBO* fbo_transparency;
	FBO* fbo_transparency2;
	FBO* fbo_shadow;

	GLuint tex_font;
	GLuint dl_font;

};
