#pragma once
#include "View.h"
#include <map>
#include <boost/shared_ptr.hpp>

/*!
 * This class defines a item of the strip.
 * @author Michael Beham
 * @date 3.6.2011
 */
class Strip
{
public:
	typedef boost::shared_ptr<Strip> StripPtr;

	/*!
	 * standard construtor
	 */
	Strip();
	/*!
	 * standard deconstructor
	 */
	~Strip(); 
	
	unsigned int GetNumberOfItems();
	/*!
	 * This method returns the item at the specified position
	 * @param at [int] position of the item.
	 * @return [VertexPtr] Vertex Item
	 */
	VertexPtr GetItemAt(int at);
	
	void AddItem(VertexPtr item);
	/*!
	 * This method sets a item at a specified position
	 * @param item [VertexPtr] item of the item
	 * @param pos [int] position of th item. 
	 */
	void SetItemAtt(VertexPtr item, int pos);
	
	/*!
	 * This method removes the last item in the strip
	 */
	void RemoveLastItem();
	/*!
	 * This Method removes the item at a specified position.
	 * @param at [int] position in the strip
	 */
	void RemoveItemAt(int at);
	/*!
	 * This Method removes the VertexPtr.
	 * @param item [VertexPtr] item of the vertex
	 */
	void RemoveItemAt(VertexPtr item);


private:	
	VertexPtr m_owner;
	std::vector<VertexPtr> m_items;
};

/*!
 * This class defines the Map of the Treemap
 * @author Michael Beham
 * @date 3.6.2011
 */
class MapVertex
{
public:
	typedef boost::shared_ptr<Strip> StripPtr;
	typedef boost::shared_ptr<MapVertex> MapVertexPtr;

	/*!
	 * standard constructor
	 */
	MapVertex(VertexPtr owner, double width, double height);

	/*!
	 * Returns the width of this strip
	 * @return [double] width of the strip
	 */
	double GetWidth();
	/*!
	 * Returns the height of this strip
	 * @return [double] height of the strip
	 */
	double GetHeight();

	/*!
	 * This method sets the width of the strip
	 * @param width [double] width of the strip
	 */
	void SetWidth(double width);
	/*!
	 * This method sets the height of the strip
	 * @param height [double] height of the strip
	 */
	void SetHeight(double height);

	/*!
	 * This method calculates the average aspect radio of the strip
	 * @return [double] average aspect ratio.
	 */
	double CalculateAvarangeAspectRatio();
	/*!
	 * This method calculates the last aspect radio of the strip
	 * @return [double] average aspect ratio.
	 */
	double CalculateLastAspectRatio();
	/*!
	 * This method calculates the aspect radio of the specified strip.
	 * @return [double] average aspect ratio.
	 */
	double CalculateAspectRatioOfStrip(int at);

	/*!
	 * This method returns the width of a specfied strip
	 * @param at [unsigned int] selected strip
	 * @return [double] width of the strip
	 */
	double GetWidthOfItemsAtStrip(unsigned int at);
	/*!
	 * This method returns the width of the last strip
	 * @return [double] width of the strip
	 */
	double GetWidthOfItemsAtLastStrip();
	/*!
	 * This method returns the height of the strips.
	 * @return [double] height of the strip
	 */
	double GetHeightOfItems();

	/*!
	 * This method returns the number of strips
	 * @return [unsigned int] number of strips
	 */
	unsigned int GetNumberOfStrips();
	/*!
	 * This method add a strip
	 * @param [StringPtr] strip which should be added
	 */
	void AddStrip(StripPtr strip);
	/*!
	 * This method sets at a specified position the strip
	 * @param strip [StripPtr] strip vertex which should be added.
	 * @param at [unsigned int] position 
	 */
	void SetStripAt(StripPtr strip, unsigned int at);
	/*!
	 * Returns the strip at a specfied position
	 * @param at [unsigned int] position at
	 * @return [StripPtr] Stripvertex
	 */
	StripPtr GetStripAt(unsigned int at);
	/*!
	 * Returns the last strip element
	 * @return [StripPtr] return the last strip
	 */
	StripPtr GetLastStrip();
	/*!
	 * Remove the strip at a specified position
	 * @param at [int] specififed position.
	 */
	void RemoveStripAt(int at);

	/*!
	 * Returns the Owner of this element
	 * @return [VertexPtr] returns the owner
	 */
	VertexPtr GetOwner();
	/*!
	 * This method sets the owner
	 * @param [VertexPtr] owner of the owner
	 */
	void SetOwner(VertexPtr owner);


private:
	double m_width;
	double m_height;
	VertexPtr m_owner;
	std::vector<StripPtr> m_strips;
};

/*!
 * This class creates the treemap layout. 
 * We uses the algorithm: "Slice and Dice" and "Strips".
 * For more information read:
 * - http://www.inf.uni-konstanz.de/algo/lehre/ws08/projekt/ausarbeitungen/nocaj.pdf
 * - http://hcil.cs.umd.edu/trs/2001-18/2001-18.html
 * @author Michael Beham
 * @date 3.6.2011
 */
class TreeMap :	public View
{
public:
	typedef boost::shared_ptr<Strip> StripPtr;
	typedef boost::shared_ptr<MapVertex> MapVertexPtr;

	/*! 
	 * standard constructor
	 */
	TreeMap(int type);
	/*!
	 * standard deconstructor
	 */
	~TreeMap(void);

	/*!
	 * This method calculates the layout
	 */
	virtual void Calculate();

	/*!
	 * The vertex positions of the layout
	 */
	std::map<unsigned int, MapVertexPtr> m_mapVertex;
private:
	void CalculateStripSliceAndDice(MapVertexPtr actStrip, bool horizontal);
	void CalculateStrip(MapVertexPtr actstrip);
	int m_type;

};
