#pragma once
//#include "HierarchyEdge.h"
//#include "RelationEdge.h"
#include "GUIVertex.h"
#include <memory>
#include <vector>
#include <iostream>
#include <algorithm>
#include <boost/shared_ptr.hpp>

using glm::vec3;

/*!
 * GUI Vertex Class
 * Diese Klasse definiert ein Node Objekt des Trees, welche Methoden fr Operationen im Tree zur Verfgung stellt. 
 * Man muss anmerken, dass diese Klasse nichts fr das Rendering speichert, da dies in die Klasse GUIVertex ausgelagert ist.
 */
class Vertex
{
public:
	typedef boost::shared_ptr<Vertex> VertexPtr;

	/*!
	 * Standardkonstruktor
	 */
	Vertex(void);
	/*!
	 * Konstruktor, welcher ein Vertex mit einer id und Namen initialisiert.
	 * @param [int] id ... Id des Vertex
	 * @param [std::string] name... Name des Vertex
	 */
	Vertex(unsigned int id, std::string name);
	/*!
	 * Dekonstruktor
	 */
	~Vertex(void);

	//void SetID(unsigned int id);	//Die Id sollte eigentlich immer gleich dem Key in der Map sein. 
	/*!
	 * Diese MEthode liefert die ID des Vertex
	 * @return [unsigned int] ID des Vertex
	 */
	unsigned int GetID();
	/*!
	 * Diese Methode setzt den Namen des Vertex
	 * @param [std::string] name Name des Vertex
	 */
	void SetName(std::string name);
	/*!
	 * Diese Methode liefert den Namen des Vertex
	 * @return [string] Name des Vertex
	 */
	std::string GetName();
	/*!
	 * Diese Methode setzt das Parentnode des Vertex
	 * @param [VertexPtr] parent ... Parentobjekt
	 */
	void SetParentVertex(VertexPtr parent);
	/*!
	 * Diese Methode liefert den Parentnode des aktuellen Vertexobjekt
	 * @return [VertexPtr] ... Parentobj.
	 */
	VertexPtr GetParent();
	/*!
	 * Diese Methode fgt ein Kindobjekt zum Vertex hinzu.
	 * @param [VertexPtr] parent ... Childelement
	 */
	void AddChildVertex(VertexPtr parent);
	/*!
	 * Diese Methode liefert die Anazhl der Childvertices zum aktuellen Childobjekt
	 * @return [unsigned int] Anzahl der Childelemente.
	 */
	unsigned int GetNumberChildVertices();
	/*!
	 * Diese Methode liefert alle Childvertices zu diesem Vertexobjekt
	 * @retun [std::vector<VertexPtr>] Alle Childvertices zum Knoten.
	 */
	std::vector<VertexPtr> GetChildVertices();
	/*!
	 * Diese Methode liefert das Childvertex zur angegebenen Position
	 * @param [int] at ... Position des Vertex
	 * @return [VertexPtr] Das gewhlte Childvertex
	 */
	VertexPtr GetChildVertexAt(int at);
	//void AddRelationEdge(RelationEdge& edge);
	
	/*!
	 * Diese Methode setzt die Hierarchy des Vertex
	 * @param [unsigned int] level Gewhlte Hierarchyebene
	 */
	void SetHierarchyLevel(unsigned int level);
	/*!
	 * Diese Methode liefert die Hierarchyebene des Vertex
	 * @return [unsigned int] aktuelle Hierarchyebene.
	 */
	unsigned int GetHierarchyLevel();

	/*!
	 * Diese Methode setzt den Pfad zum Rootknoten
	 * @param [std::vector<unsigned int>] pathToRoot Pfad zum Rootknoten
	 */
	void SetPathToRoot(std::vector<unsigned int> pathToRoot);
	/*!
	 * Diese Methode liefert den Pfad zum Rootknoten
	 * @return [std::vector<unsigned int>] Pfad zum Rootknoten
	 */
	std::vector<unsigned int> GetPathToRoot();

	bool operator<=(VertexPtr v);

	bool operator<(VertexPtr v);

	bool operator()(Vertex v1,Vertex v2);

	bool operator()(VertexPtr v1,VertexPtr v2);

protected:
	unsigned int m_id;
	std::string m_name;

	VertexPtr m_parent;
	std::vector<VertexPtr> m_childs;

	std::vector<unsigned int> m_pathToRoot;

	//std::vector<HierarchyEdge&> m_hierarchyedge;
	//std::vector<RelationEdge&> m_relationedge;

	unsigned int m_level;
};
